/*
* ----------------------------------------------------------------------------
* "THE BEER-WARE LICENSE" (Revision 42):
* <phk@FreeBSD.ORG> wrote this file. As long as you retain this notice you
* can do whatever you want with this stuff. If we meet some day, and you think
* this stuff is worth it, you can buy me a beer in return Daniel Cogo de Vargas
* and Ivon Eduardo Esser Rosa.
* ----------------------------------------------------------------------------
*/

/*
 * Personagem.java
 *
 * Created on 20/10/2009, 22:17:56
 */

package elementos;

import java.awt.Image;
import java.awt.Rectangle;
import javax.swing.ImageIcon;
import javax.swing.JPanel;

/**
 * Classe Personagem contém o mínimo necessário para adicionar
 * um objeto da Tela e fazer com que este mova por toda sua
 * extensão.
 *
 * @author Emerson Ribeiro de Mello
 */
public abstract class Personagem {

    private Image imagem; // onde será carregada a imagem do jogador (usado pela Tela)
    protected JPanel tela; // para interagir com o JPanel onde está inserido
    protected int x; // coordenada X atual do jogador
    protected int y;  // coordenada Y atual do jogador
    protected int largura; // largura da imagem que fora carregada
    protected int altura; // altura da imagem que fora carregada
    protected int dirX; // direção no eixo X que o objeto deverá se movimentar
    protected int dirY; // direção no eixo Y que o objeto deverá se movimentar
    protected int vida;

    /**
     * * É necessário informar o nome da imagem que será carregada no jogador
     * @param nomeImagem caminho onde está localizada a imagem
     * @param jp referencia ao JPanel pai onde este objeto jogador está inserido
     * @param px posicao X inicial onde o jogador irá aparecer
     * @param py posicao Y inicial onde o jogador irá aparecer
     */

    // metodo construtor de personagem
    public Personagem(String nomeImagem, JPanel jp, int px, int py, int vd) {
        ImageIcon iicon = new ImageIcon(this.getClass().getResource("/resources/"+nomeImagem));
        this.imagem = iicon.getImage();

        this.vida = vd;
        this.tela = jp;
        this.x = px;
        this.y = py;
        this.largura = imagem.getWidth(null);
        this.altura = imagem.getHeight(null);
    }

    /**
     * Informa nova imagem para o Personagem caso necessário.
     * @param im - Nome da imagem que deverá estar dentro do diretório Resources.
     */
   // metodo para selecionar e chamar personagem
    public void alterarImagem(String im){ // invoca o personagem escolhido
        ImageIcon iicon = new ImageIcon(this.getClass().getResource("/resources/"+im));
        this.imagem = iicon.getImage();
    }
    /**
     * Método responsável pela movimentação do objeto jogador na tela.
     * Irá alterar os valores atuais dos atributos x e y.
     */
    public void mover() { // metodo para mover personagem
        // obtendo a largura e altura da tela
        int lTela = this.tela.getSize().width;
        int aTela = this.tela.getSize().height;

        //teste para evitar que ultrapasse os limites da tela - eixo X
        x += (((dirX > 0) && (x + dirX < lTela - this.largura)) || ((dirX < 0) && (x - dirX > 0))) ? dirX : 0;

        //teste para evitar que ultrapasse os limites da tela - eixo Y
        y += (((dirY > 0) && (y + dirY < aTela - this.altura)) || ((dirY < 0) && (y - dirY > 0))) ? dirY : 0;
    }




   

    /**
     * Retorna a imagem para que possa ser desenhada na Tela
     * @return
     */
    public Image getImagem() { // retorna imagem
        return imagem;
    }

    /**
     * Retorna a coordenada X para desenhar na posição correta na Tela
     * @return
     */
    public int getX() { // retorna posição x
        return x;
    }

    /**
     * Retorna a coordenada X para desenhar na posição correta na Tela
     * @return
     */
    public int getY() { // retorna posição y
        return y;
    }

    /**
     * Definindo uma nova coordenada X para o objeto
     * @param x posicao nova
     */
    public void setX(int x) { 
        this.x = x;
    }

    /**
     * Definindo uma nova coordenada y para o objeto
     * @param y posicao nova
     */
    public void setY(int y) {
        this.y = y;
    }

    /**
     * Para incrementar/decrementar a atual coordenada
     * @param ix valor positivo, negativo ou zero
     */
    public void incrementaX(int ix) {
        this.x += ix;
    }

    /**
     * Para incrementar/decrementar a atual coordenada
     * @param ix valor positivo, negativo ou zero
     */
    public void incrementaY(int iy) {
        this.y += iy;
    }

    /**
     * Retorna a largura da imagem
     * @return largura
     */
    public int getLargura() {
        return largura;
    }

    /**
     * Retorna a altura da imamge
     * @return
     */
    public int getAltura() {
        return altura;
    }

    /**
     * Usado para detecção de colisões no jogo
     * @return Cria um retangulo com a posicao atual do jogador e com as dimensoes da
     * imagem que fora carregada para esse jogador.
     */
    public Rectangle getLimites() {
        return new Rectangle(x, y, largura, altura);
    }

    /**
     * Retorna nível atual de vida do Jogador
     * @return
     */
    public int obtemVida(){
        return vida;
    }

    /**
     * Define vida do jogador atual com o valor "v"
     * @param v - Quantidade de vida a ser definida
     */
    public void defineVida(int v){
        this.vida = v;
    }
}
