/*
* ----------------------------------------------------------------------------
* "THE BEER-WARE LICENSE" (Revision 42):
* <phk@FreeBSD.ORG> wrote this file. As long as you retain this notice you
* can do whatever you want with this stuff. If we meet some day, and you think
* this stuff is worth it, you can buy me a beer in return Daniel Cogo de Vargas
* and Ivon Eduardo Esser Rosa.
* ----------------------------------------------------------------------------
*/

/*
 * Tela.java
 *
 * Created on 20/10/2009, 22:17:56
 */
package jogo;

import elementos.*;
import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.util.ArrayList;
import java.util.Random;
import javax.swing.JPanel;
import javax.swing.Timer;

/**
 * Veja: http://zetcode.com/tutorials/javagamestutorial
 * Veja: http://www.cokeandcode.com/info/tut2d.html
 * 
 * @author Emerson Ribeiro de Mello
 */
public class Tela extends JPanel implements ActionListener {

    private Timer timer; // para ficar desenhando a Tela2 de tempos em tempos
    private Jogador j1; // para ilustrar o uso da classe Personagem
    private Jogador j2; // para ilustrar o uso da classe Personagem
    private Inimigo inim;
    private Random r; // só para colocar os jogadores em posições aleatórias na tela
    public boolean jogoAtivo; // para saber se o jogo está rodando
    public boolean inicial; // para saber se é um novo jogo
    public Principal pai;
    private int movInimigo; // um contador para uma simples movimentacao dos inimigos
    private ArrayList<Inimigo> vetorInimigos;
    private int velocidade;

    public Tela(Principal p) {

        //Criação de vetor de inimigos
        vetorInimigos = new ArrayList<Inimigo>();

        // Guardando um referencia ao JFrame PAI para pegar e modificar
        // alguns atributos que lá estão
        this.pai = p;

        this.r = new Random();
        this.movInimigo = 0;


        // atributos que poderão ser modificados pela classe Principal
        this.jogoAtivo = true;
        this.inicial = false;

        // trata eventos das teclas pressionadas
        this.addKeyListener(new TratadorEventoTeclado());

        /* definindo algumas propriedades do JPanel */
        this.setFocusable(true); // assume foco
        this.setBackground(Color.BLACK); // alterando a cor
        this.setDoubleBuffered(true); // tornando o redesenho mais rapido


        /* Irá atualizar a tela a cada 5 milisegundos */
        this.timer = new Timer(5, this);
        this.timer.start();
    }

    /**
     * É chamado sempre que começar uma nova rodada
     */
    public void novaRodada() {
        /* criando 2 jogadores e posicionando-os na tela de forma aleatória */
        /*int px = r.nextInt(this.getWidth() - 50);
        int py = r.nextInt(this.getHeight() - 50);*/

        // limpar o vetorInimigos
        vetorInimigos.clear();

        //define velocidade inicial dos inimigos
        velocidade = 10;

        // setando barra de vida para 100 (barra cheia)
        pai.jPBVida.setValue(100);
        // Setando barra de poder para zero
        pai.jPBPoder.setValue(100);

        criaJogador();

        /* int px = r.nextInt(this.getWidth() - 50); */

        int py = r.nextInt(this.getHeight() - 50);
        int vd = r.nextInt(9) + 1;
        this.j2 = new Inimigo(this, 550, py, vd);


        criaInimigos();


    }

    public void criaJogador() {
        // testes para podr imprimir a imagem do personagem na tela
        if (pai.jogador.equals("penguin.png")) {
            this.j1 = new Tux(this, 10, 120, 100);
        } else {
            if (pai.jogador.equals("Duke.png")) {
                this.j1 = new Duke(this, 10, 120, 100);
            } else {
                if (pai.jogador.equals("Puffy.png")) {
                    this.j1 = new Puffy(this, 10, 120, 100);
                }
            }
        }
    }

    /**
     * Irá desenhar os objetos na tela
     * Será chamado a cada 5 milisegundos, ou seja, a tela está em constante
     * atualização
     * @param g
     */
    @Override
    public void paint(Graphics g) {
        super.paint(g);
        Graphics2D g2d = (Graphics2D) g;

        if (jogoAtivo) { // o jogo está rodando!!!
            // Jogo acabou de ser carregado, mas ainda não foi iniciado um novo jogo
            // para criar os objetos somente 1 vez.
            if (inicial == true) {
                this.novaRodada();
                inicial = false;
            }

            for (Inimigo i : vetorInimigos) {
                g2d.drawImage(i.getImagem(), i.getX(), i.getY(), this);
                i.mover();
                for (Tiro t : i.obterTiro()) {
                    g2d.drawImage(t.getImagem(), t.getX(), t.getY(), this);
                    t.mover(-3);
                }

            }

            // para garantir que eles nao sao nulos, antes de pintá-los na tela
            if ((j1 != null)) {
                if (j1 instanceof Puffy) {
                    ((Puffy) j1).decrementar();
                }
                if (j1 instanceof Duke) {
                    ((Duke) j1).decrementar();
                }

                g2d.drawImage(j1.getImagem(), j1.getX(), j1.getY(), this);
                for (Tiro t : j1.obterTiro()) {
                    g2d.drawImage(t.getImagem(), t.getX(), t.getY(), this);
                    t.mover(3);
                }
                //g2d.drawImage(j2.getImagem(), j2.getX(), j2.getY(), this);
            }

        } else { // fim de jogo, imprima a mensagem

            // ativando o antialiasing
            g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);

            // modificando as características fonte
            Font small = new Font("Helvetica", Font.BOLD, 14);
            FontMetrics metr = this.getFontMetrics(small);
            g2d.setColor(Color.white);
            g2d.setFont(small);

            String msg = "Fim de Jogo!";

            g2d.drawString(msg, (this.getWidth() - metr.stringWidth(msg)) / 2,
                    this.getHeight() / 2);

        }

    }

    /**
     * Irá tratar todos os eventos que ocorrerem no JPanel.
     * O pressionar de uma tecla é um evento a ser tratado.
     *
     * @param e evento que ocorreu na Tela2
     */
    public void actionPerformed(ActionEvent e) {
        // para movimentar o jogador pela tela

        // somente depois que o objeto for criado, ou seja, depois de um novo jogo
        if ((j1 != null) && (jogoAtivo == true)) {

            /* Movendo a joaninha
             * 
             * A tela é atualizada a cada 5 milisegundos. A joaninha irá se movimentar
             * a cada 25 milisegundos
             */
            movInimigo = (movInimigo % 15) + 1;

            if (movInimigo == 15) {
                this.movimentarInimigos();
            }

            // movendo o pinguim
            j1.mover();
            // para verificar se houve alguma colisao entre os jogadores
            this.verificaColisoes();
            // pintando a tela novamente
            repaint();
        }
    }

    /**
     * Fazendo a joaninha andar
     * Uma lógica simples e nada inteligente para movimentar
     */
    private void movimentarInimigos() {

        int ondex = r.nextInt(30);
        int ondey = r.nextInt(30);
        boolean tiros = r.nextBoolean(); //se irá incrementar ou decrementar
        boolean increY = r.nextBoolean(); //se irá incrementar ou decrementar


                int pontos = new Integer(pai.jLPontos.getText()).intValue();
                if ((pontos % 6 == 0)&& (pontos != 0)){ // se a pontuação do jogador for maior que 6, incrementa velocidade do movimento do inimigo
                    velocidade ++;
                }

        for (Inimigo i : vetorInimigos) {

            i.Atirar();

            for (int ux = 0; ux < velocidade; ux++) { //Definição do movimento do inimigo em x
                for (int uy = 0; uy < 2; uy++) { // Definição do movimento do inimigo em y
                    if (i.getY() + i.getAltura() + 1 < this.getHeight()) { //se nao tiver ultrapassado a tela, incrementa
                        i.incrementaY(1);
                    } else { // se ultrapassou a tela, volta para o início em y.
                        i.setY(1);
                    }

                }
                if (i.getX() - 1 > 0) { // se não ultrapassar o limite da tela
                    i.incrementaX(-1);
                } else {
                    i.setX(this.getWidth() - i.getLargura());
                }
            }

        }
    }

    /** for (Inimigo i : vetorInimigos){
    // se true, ele vai incrementar o valor sorteado em ondex - eixo X;
    if (increX == true) {
    if (i.getX() + i.getLargura() + ondex < this.getWidth()) {
    i.incrementaX(ondex);
    } else {
    i.setX(r.nextInt(this.getWidth() - i.getLargura()));
    }
    } else { // se nao, ele vai decrementar
    if (i.getX() - ondex > 0) { // se não ultrapassar o limite da tela
    i.incrementaX(ondex * -1);
    } else {
    i.setX(this.getWidth() - i.getLargura());
    }
    }

    // se true, ele vai incrementar o valor sorteado em ondey - eixo Y;
    if (increY == true) {
    if (i.getY() + i.getAltura() + ondey < this.getHeight()) {
    i.incrementaY(ondey);
    } else {
    i.setY(this.getHeight() - i.getAltura());
    }
    } else { // se nao, ele vai decrementar
    if (i.getY() - ondey > 0) { // se não ultrapassar o limite da tela
    i.incrementaY(ondey * -1);
    } else {
    i.setY(this.getHeight() - i.getAltura());
    }
    }
    }
    }
     *
     * /
    /**
     * Verifica se houve colisoes entre os objetos na tela.
     */
    private void verificaColisoes() {
        //Criação de Array List auxiliar
        ArrayList<Inimigo> auxI = new ArrayList<Inimigo>();
        ArrayList<Tiro> auxT = new ArrayList<Tiro>();

        // Para verificar o array list de inimigos
        verificaInimigos();

        // descobrindo as dimensoes e posicao dos jogadores
        for (Inimigo i : vetorInimigos) {

            ArrayList<Tiro> auxTi = new ArrayList<Tiro>();


            Rectangle rJ1 = j1.getLimites();
            Rectangle rJ2 = i.getLimites();



            // usufruindo do método intersects da classe Rectangle para verificar
            // a colisão
            if (rJ1.intersects(rJ2)) { // se personagem colidir com adversario
                auxI.add(i);
                if (j1 instanceof Puffy) { // testa se puffy estiver com escudo ativo
                    if (((Puffy) j1).escudoAtivo == false) {
                        int v = pai.jPBVida.getValue();
                        if (v > 50) {
                            int nv = v - 50;
                            pai.jPBVida.setValue(nv);
                        } else {
                            pai.jPBVida.setValue(0);
                            this.jogoAtivo = false;
                        }
                        //logicaDoJogo();
                    }
                    int pontos = new Integer(pai.jLPontos.getText()).intValue();
                    pontos++;
                    pai.jLPontos.setText("" + pontos);
                }
                if (j1 instanceof Duke) { // testa se duke estiver com escudo ativo
                    if (((Duke) j1).escudoAtivo == false) {
                        int v = pai.jPBVida.getValue();
                        if (v > 50) {
                            int nv = v - 50;
                            pai.jPBVida.setValue(nv);
                        } else {
                            pai.jPBVida.setValue(0);
                            this.jogoAtivo = false;
                        }
                        //logicaDoJogo();
                    }
                    int pontos = new Integer(pai.jLPontos.getText()).intValue();
                    pontos++;
                    pai.jLPontos.setText("" + pontos);
                }
                if (j1 instanceof Tux) { //para o tux
                    int v = pai.jPBVida.getValue();
                    if (v > 50) {
                        int nv = v - 50;
                        pai.jPBVida.setValue(nv);
                    } else {
                        pai.jPBVida.setValue(0);
                        this.jogoAtivo = false;
                    }
                    int pontos = new Integer(pai.jLPontos.getText()).intValue();
                    pontos++;
                    pai.jLPontos.setText("" + pontos);

                }


            }


            // Caso tiro acerte jogador.
            for (Tiro t : i.obterTiro()) { // detecta colisão entre jogaor e tiro
                Rectangle rT = t.getLimites();
                if (rT.intersects(rJ1)) {
                    if (j1 instanceof Puffy) { // testa se puffy estiver com escudo ativo
                        if (((Puffy) j1).escudoAtivo == false) {
                            auxTi.add(t);
                            int v = pai.jPBVida.getValue();
                            if (v > 0) {
                                v--;
                                pai.jPBVida.setValue(v);
                                //retiraVida();
                            } else {
                                pai.jPBVida.setValue(0);
                                this.jogoAtivo = false;
                            }
                        }
                        if (((Puffy) j1).escudoAtivo == true) {
                            auxTi.add(t);
                        }
                    }
                    if (j1 instanceof Duke) { // testa se duke estiver com escudo ativo
                        if (((Duke) j1).escudoAtivo == false) {
                            auxTi.add(t);
                            int v = pai.jPBVida.getValue();
                            if (v > 0) {
                                v--;
                                pai.jPBVida.setValue(v);
                                //retiraVida();
                            } else {
                                pai.jPBVida.setValue(0);
                                this.jogoAtivo = false;
                            }
                        }
                        if (((Duke) j1).escudoAtivo == true) {
                            auxTi.add(t);
                        }
                    }
                    if (j1 instanceof Tux) { // para tux
                        auxTi.add(t);
                        int v = pai.jPBVida.getValue();
                        if (v > 0) {
                            v--;
                            pai.jPBVida.setValue(v);
                            //retiraVida();
                            } else {
                            pai.jPBVida.setValue(0);
                            this.jogoAtivo = false;
                        }
                    }



                }

                if (t.getX() - 1 < 0) {
                    auxTi.add(t);
                }

            }



            for (Tiro t : auxTi) {
                i.obterTiro().remove(t);
            }

            // Caso tiro acerte adversario
            for (Tiro t : j1.obterTiro()) {
                Rectangle rT = t.getLimites();
                if (rT.intersects(rJ2)) { // testa se colide com tiro 
                    //se tiro acertar a joaninha
                    //int c = pai.jPBPoder.getValue();
                    //c = c + 5;
                    //if (c < 100) {
                    //    pai.jPBPoder.setValue(c);
                    //}
                    //if (c > 95){
                    //    pai.jPBPoder.setValue(100);
                    //}

                    int vida = i.obtemVida();
                    vida--;
                    i.defineVida(vida);

                    auxT.add(t);
                    // caso tiro acerte adversario incrementa poder
                    int v = i.obtemVida();
                    if (v <= 0) { // testa limites da variavel poder
                        auxI.add(i);
                        int c = pai.jPBPoder.getValue();
                        c = c + 5; // cada adiversario morto aumenta 5 na quantia de poder do personagem
                        if (c < 100) {
                            pai.jPBPoder.setValue(c);
                        }
                        if (c > 95) {
                            pai.jPBPoder.setValue(100);
                        }
                    }

                }

                if (t.getX() + t.getLargura() + 1 > this.getWidth()) {
                    auxT.add(t);
                }

            }
        }

        for (Inimigo i : auxI) { // caso tiro colidir com adversario
            vetorInimigos.remove(i); // remove adversario 
            int pontos = new Integer(pai.jLPontos.getText()).intValue();
            pontos++; // aumenta pontos do jogador
            pai.jLPontos.setText("" + pontos);

        }
        for (Tiro t : auxT) { // remove tiro
            j1.obterTiro().remove(t);

        }
    }
// criação de um arrei auxiliar de inimigos para poder realizar a exclusão dos mesmos
    public void excluir() {
        ArrayList<Inimigo> auxI = new ArrayList<Inimigo>();
        ArrayList<Tiro> auxTi = new ArrayList<Tiro>();

        
        for (Inimigo i : vetorInimigos) {
            auxI.add(i);
            int pontos = new Integer(pai.jLPontos.getText()).intValue();
            pontos++;
            pai.jLPontos.setText("" + pontos);

            for (Tiro t : i.obterTiro()) {
                auxTi.add(t);
            }
            for (Tiro t : auxTi) {
                i.obterTiro().remove(t);
            }

        }
        for (Inimigo i : auxI) {
            vetorInimigos.remove(i);
        }


    }
    // se não tiver mais nenhum inimigo na tela, sera criado um novo vetor de inimigo
    private void verificaInimigos() {
        int num = vetorInimigos.size();

        if (num == 0) {
            criaInimigos();
        }

    }

    /**
     * Criando vetor de inimigos, as devidas posições x e y.
     */
    private void criaInimigos() {
        for (int a = 0; a < 2; a++) { 
            int py = r.nextInt(this.getHeight() - 50);
            int vd = r.nextInt(9) + 1;
            vetorInimigos.add(new Inimigo(this, 550, py, vd));
        }
        for (int a = 0; a < 3; a++) {
            int py = r.nextInt(this.getHeight() - 50);
            int vd = r.nextInt(9) + 1;
            vetorInimigos.add(new Inimigo(this, 500, py, vd));
        }
    }

    /**
     * Uma lógica simples para o jogo, cujo objetivo é fazer com que o pinguim
     * ao passar por cima de uma joaninha, aumente o número de pontos e faça
     * nascer uma joaninha em outro local na tela
     */
    public void logicaDoJogo() {

        // incrementando o numero de pontos do jogador
        int pontos = new Integer(pai.jLPontos.getText()).intValue();
        pontos++;

        pai.jLPontos.setText("" + pontos);

        // fazendo a joaninha 'nascer' em outra parte da tela
        int px = r.nextInt(this.getWidth() - j2.getLargura()); // aletorio dentro do limite da tela
        int py = r.nextInt(this.getHeight() - j2.getAltura()); // aletorio dentro do limite da tela
        j2.setX(px);
        j2.setY(py);

        /*
         * O objetivo do jogo é marcar 10 pontos. Quando isso for atingido. FIM!
         */
        if (pontos == 10) {
            jogoAtivo = false;
        }




    }

    /**
     * Classe interna e privada para tratar os eventos de tecla pressionada
     * e tecla solta.
     * Neste exemplo, sempre que uma tecla for pressionada, o j1 irá se mover
     *
     * Uma classe interna consegue acessar todos os atributos (mesmo os privados)
     * da classe mais externa que a contém.
     *
     */
    private class TratadorEventoTeclado extends KeyAdapter {

        /**
         * Quando ocorrer o evento de tecla solta, será invocado o método teclaSolta
         * do objeto j1
         * @param e evento do teclado
         */
        @Override
        public void keyReleased(KeyEvent e) {
            if (j1 != null) {
                j1.teclaSolta(e);
            }
        }

        /**
         * Quando ocorrer o evento de tecla pressionada, será invocado o
         * método teclaPressionada do objeto j1
         * @param e evento do teclado
         */
        @Override
        public void keyPressed(KeyEvent e) {
            if (j1 != null) {
                j1.teclaPressionada(e);
            }
        }
    }
}
